import pytest


@pytest.mark.physical_camera
def test_image_context_manager(physical_camera_frame_2d):
    import zivid

    with physical_camera_frame_2d.image_rgba() as image:
        assert image is not None
        assert isinstance(image, zivid.Image)


@pytest.mark.physical_camera
def test_image(physical_camera_frame_2d):
    import zivid

    image = physical_camera_frame_2d.image_rgba()
    assert image is not None
    assert isinstance(image, zivid.Image)


@pytest.mark.physical_camera
def test_state(physical_camera_frame_2d):
    import zivid

    state = physical_camera_frame_2d.state
    assert state is not None
    assert isinstance(state, zivid.CameraState)


@pytest.mark.physical_camera
def test_info(physical_camera_frame_2d):
    import zivid

    info = physical_camera_frame_2d.info
    assert info is not None
    assert isinstance(info, zivid.FrameInfo)


@pytest.mark.physical_camera
def test_settings(physical_camera_frame_2d):
    import zivid

    settings_2d = physical_camera_frame_2d.settings
    assert settings_2d is not None
    assert isinstance(settings_2d, zivid.Settings2D)


@pytest.mark.physical_camera
def test_release(physical_camera_frame_2d):
    physical_camera_frame_2d.image_rgba()
    physical_camera_frame_2d.release()
    with pytest.raises(RuntimeError):
        physical_camera_frame_2d.image_rgba()


@pytest.mark.physical_camera
def test_context_manager(physical_camera):
    import zivid

    settings_2d = zivid.Settings2D(acquisitions=[zivid.Settings2D.Acquisition()])
    with physical_camera.capture(settings_2d) as frame_2d:
        frame_2d.image_rgba()
    with pytest.raises(RuntimeError):
        frame_2d.image_rgba()



"""Auto generated, do not edit."""
# pylint: disable=too-many-lines,protected-access,too-few-public-methods,too-many-arguments,line-too-long,missing-function-docstring,missing-class-docstring,too-many-branches,too-many-boolean-expressions
import _zivid


class CameraState:
    class Temperature:
        def __init__(
            self,
            dmd=_zivid.CameraState.Temperature.DMD().value,
            general=_zivid.CameraState.Temperature.General().value,
            led=_zivid.CameraState.Temperature.LED().value,
            lens=_zivid.CameraState.Temperature.Lens().value,
            pcb=_zivid.CameraState.Temperature.PCB().value,
        ):

            if isinstance(
                dmd,
                (
                    float,
                    int,
                ),
            ):
                self._dmd = _zivid.CameraState.Temperature.DMD(dmd)
            else:
                raise TypeError(
                    "Unsupported type, expected: (float, int,), got {value_type}".format(
                        value_type=type(dmd)
                    )
                )

            if isinstance(
                general,
                (
                    float,
                    int,
                ),
            ):
                self._general = _zivid.CameraState.Temperature.General(general)
            else:
                raise TypeError(
                    "Unsupported type, expected: (float, int,), got {value_type}".format(
                        value_type=type(general)
                    )
                )

            if isinstance(
                led,
                (
                    float,
                    int,
                ),
            ):
                self._led = _zivid.CameraState.Temperature.LED(led)
            else:
                raise TypeError(
                    "Unsupported type, expected: (float, int,), got {value_type}".format(
                        value_type=type(led)
                    )
                )

            if isinstance(
                lens,
                (
                    float,
                    int,
                ),
            ):
                self._lens = _zivid.CameraState.Temperature.Lens(lens)
            else:
                raise TypeError(
                    "Unsupported type, expected: (float, int,), got {value_type}".format(
                        value_type=type(lens)
                    )
                )

            if isinstance(
                pcb,
                (
                    float,
                    int,
                ),
            ):
                self._pcb = _zivid.CameraState.Temperature.PCB(pcb)
            else:
                raise TypeError(
                    "Unsupported type, expected: (float, int,), got {value_type}".format(
                        value_type=type(pcb)
                    )
                )

        @property
        def dmd(self):
            return self._dmd.value

        @property
        def general(self):
            return self._general.value

        @property
        def led(self):
            return self._led.value

        @property
        def lens(self):
            return self._lens.value

        @property
        def pcb(self):
            return self._pcb.value

        @dmd.setter
        def dmd(self, value):
            if isinstance(
                value,
                (
                    float,
                    int,
                ),
            ):
                self._dmd = _zivid.CameraState.Temperature.DMD(value)
            else:
                raise TypeError(
                    "Unsupported type, expected: float or  int, got {value_type}".format(
                        value_type=type(value)
                    )
                )

        @general.setter
        def general(self, value):
            if isinstance(
                value,
                (
                    float,
                    int,
                ),
            ):
                self._general = _zivid.CameraState.Temperature.General(value)
            else:
                raise TypeError(
                    "Unsupported type, expected: float or  int, got {value_type}".format(
                        value_type=type(value)
                    )
                )

        @led.setter
        def led(self, value):
            if isinstance(
                value,
                (
                    float,
                    int,
                ),
            ):
                self._led = _zivid.CameraState.Temperature.LED(value)
            else:
                raise TypeError(
                    "Unsupported type, expected: float or  int, got {value_type}".format(
                        value_type=type(value)
                    )
                )

        @lens.setter
        def lens(self, value):
            if isinstance(
                value,
                (
                    float,
                    int,
                ),
            ):
                self._lens = _zivid.CameraState.Temperature.Lens(value)
            else:
                raise TypeError(
                    "Unsupported type, expected: float or  int, got {value_type}".format(
                        value_type=type(value)
                    )
                )

        @pcb.setter
        def pcb(self, value):
            if isinstance(
                value,
                (
                    float,
                    int,
                ),
            ):
                self._pcb = _zivid.CameraState.Temperature.PCB(value)
            else:
                raise TypeError(
                    "Unsupported type, expected: float or  int, got {value_type}".format(
                        value_type=type(value)
                    )
                )

        def __eq__(self, other):
            if (
                self._dmd == other._dmd
                and self._general == other._general
                and self._led == other._led
                and self._lens == other._lens
                and self._pcb == other._pcb
            ):
                return True
            return False

        def __str__(self):
            return str(_to_internal_camera_state_temperature(self))

    def __init__(
        self,
        available=_zivid.CameraState.Available().value,
        connected=_zivid.CameraState.Connected().value,
        temperature=None,
    ):

        if isinstance(available, (bool,)):
            self._available = _zivid.CameraState.Available(available)
        else:
            raise TypeError(
                "Unsupported type, expected: (bool,), got {value_type}".format(
                    value_type=type(available)
                )
            )

        if isinstance(connected, (bool,)):
            self._connected = _zivid.CameraState.Connected(connected)
        else:
            raise TypeError(
                "Unsupported type, expected: (bool,), got {value_type}".format(
                    value_type=type(connected)
                )
            )

        if temperature is None:
            temperature = self.Temperature()
        if not isinstance(temperature, self.Temperature):
            raise TypeError("Unsupported type: {value}".format(value=type(temperature)))
        self._temperature = temperature

    @property
    def available(self):
        return self._available.value

    @property
    def connected(self):
        return self._connected.value

    @property
    def temperature(self):
        return self._temperature

    @available.setter
    def available(self, value):
        if isinstance(value, (bool,)):
            self._available = _zivid.CameraState.Available(value)
        else:
            raise TypeError(
                "Unsupported type, expected: bool, got {value_type}".format(
                    value_type=type(value)
                )
            )

    @connected.setter
    def connected(self, value):
        if isinstance(value, (bool,)):
            self._connected = _zivid.CameraState.Connected(value)
        else:
            raise TypeError(
                "Unsupported type, expected: bool, got {value_type}".format(
                    value_type=type(value)
                )
            )

    @temperature.setter
    def temperature(self, value):
        if not isinstance(value, self.Temperature):
            raise TypeError("Unsupported type {value}".format(value=type(value)))
        self._temperature = value

    @classmethod
    def load(cls, file_name):
        return _to_camera_state(_zivid.CameraState(str(file_name)))

    def save(self, file_name):
        _to_internal_camera_state(self).save(str(file_name))

    def __eq__(self, other):
        if (
            self._available == other._available
            and self._connected == other._connected
            and self._temperature == other._temperature
        ):
            return True
        return False

    def __str__(self):
        return str(_to_internal_camera_state(self))


def _to_camera_state_temperature(internal_temperature):
    return CameraState.Temperature(
        dmd=internal_temperature.dmd.value,
        general=internal_temperature.general.value,
        led=internal_temperature.led.value,
        lens=internal_temperature.lens.value,
        pcb=internal_temperature.pcb.value,
    )


def _to_camera_state(internal_camera_state):
    return CameraState(
        temperature=_to_camera_state_temperature(internal_camera_state.temperature),
        available=internal_camera_state.available.value,
        connected=internal_camera_state.connected.value,
    )


def _to_internal_camera_state_temperature(temperature):
    internal_temperature = _zivid.CameraState.Temperature()

    internal_temperature.dmd = _zivid.CameraState.Temperature.DMD(temperature.dmd)
    internal_temperature.general = _zivid.CameraState.Temperature.General(
        temperature.general
    )
    internal_temperature.led = _zivid.CameraState.Temperature.LED(temperature.led)
    internal_temperature.lens = _zivid.CameraState.Temperature.Lens(temperature.lens)
    internal_temperature.pcb = _zivid.CameraState.Temperature.PCB(temperature.pcb)

    return internal_temperature


def _to_internal_camera_state(camera_state):
    internal_camera_state = _zivid.CameraState()

    internal_camera_state.available = _zivid.CameraState.Available(
        camera_state.available
    )
    internal_camera_state.connected = _zivid.CameraState.Connected(
        camera_state.connected
    )

    internal_camera_state.temperature = _to_internal_camera_state_temperature(
        camera_state.temperature
    )
    return internal_camera_state



"""Module for experimental calibration features. This API may change in the future."""

import _zivid
from zivid.camera_intrinsics import _to_camera_intrinsics


def intrinsics(camera):
    """Get intrinsic parameters of a given camera.

    Args:
        camera: A Camera instance

    Returns:
        A CameraIntrinsics instance
    """
    return _to_camera_intrinsics(
        _zivid.calibration.intrinsics(
            camera._Camera__impl  # pylint: disable=protected-access
        )
    )


def estimate_intrinsics(frame):
    """Estimate camera intrinsics for a given frame.

    This function is for advanced use cases. Otherwise, use intrinsics(camera).

    Args:
        frame: A Frame instance

    Returns:
        A CameraIntrinsics instance
    """
    return _to_camera_intrinsics(
        _zivid.calibration.estimate_intrinsics(
            frame._Frame__impl  # pylint: disable=protected-access
        )
    )



"""Contains Camera class."""
import _zivid
from zivid.frame import Frame
from zivid.frame_2d import Frame2D
from zivid.settings import Settings, _to_internal_settings
from zivid.settings_2d import Settings2D, _to_internal_settings2d
from zivid.camera_info import _to_camera_info
from zivid.camera_state import _to_camera_state


class Camera:
    """Interface to one Zivid camera.

    This class cannot be initialized directly by the end-user. Use methods on the Application
    class to obtain a Camera instance.
    """

    def __init__(self, impl):
        """Initialize Camera wrapper.

        This constructor is only used internally, and should not be called by the end-user.

        Args:
            impl:   Reference to internal/back-end instance.

        Raises:
            TypeError: If argument does not match the expected internal class.
        """
        if not isinstance(impl, _zivid.Camera):
            raise TypeError(
                "Unsupported type for argument impl. Got {}, expected {}".format(
                    type(impl), type(_zivid.Camera)
                )
            )

        self.__impl = impl

    def __str__(self):
        return str(self.__impl)

    def __eq__(self, other):
        return self.__impl == other._Camera__impl

    def capture(self, settings):
        """Capture a single frame or a single 2D frame.

        Args:
            settings: Settings to be used to capture. Can be either a Settings or Settings2D instance

        Returns:
            A Frame containing a 3D image plus metadata or a Frame2D containing a 2D image plus metadata.

        Raises:
            TypeError: If argument is neither a Settings or a Settings2D
        """
        if isinstance(settings, Settings):
            return Frame(self.__impl.capture(_to_internal_settings(settings)))
        if isinstance(settings, Settings2D):
            return Frame2D(self.__impl.capture(_to_internal_settings2d(settings)))
        raise TypeError("Unsupported settings type: {}".format(type(settings)))

    @property
    def info(self):
        """Get information about camera model, serial number etc.

        Returns:
            A CameraInfo instance
        """
        return _to_camera_info(self.__impl.info)

    @property
    def state(self):
        """Get the current camera state.

        Returns:
            A CameraState instance
        """
        return _to_camera_state(self.__impl.state)

    def connect(self):
        """Connect to the camera.

        Returns:
            Reference to the same Camera instance (for chaining)
        """
        self.__impl.connect()
        return self

    def disconnect(self):
        """Disconnect from the camera and free all resources associated with it."""
        self.__impl.disconnect()

    def write_user_data(self, user_data):
        """Write user data to camera. The total number of writes supported depends on camera model and size of data.

        Args:
            user_data: User data as 'bytes' object

        Raises:
            TypeError: Unsupported type provided for user data
        """
        if not isinstance(user_data, bytes):
            raise TypeError(
                "Unsupported type for argument user_data. Got {}, expected {}.".format(
                    type(user_data), bytes.__name__
                )
            )
        self.__impl.write_user_data(list(user_data))

    @property
    def user_data(self):
        """Read user data from camera.

        Returns:
            User data as 'bytes' object
        """
        return bytes(self.__impl.user_data)

    def release(self):
        """Release the underlying resources."""
        try:
            impl = self.__impl
        except AttributeError:
            pass
        else:
            impl.release()

    def __enter__(self):
        return self

    def __exit__(self, exception_type, exception_value, traceback):
        self.release()

    def __del__(self):
        self.release()



"""Query module version."""
from pkg_resources import get_distribution, DistributionNotFound


def get_version(module_name):
    """Return the version module version.

    Args:
        module_name: Name of a module

    Returns:
        The module version

    """
    try:
        return get_distribution(module_name).version
    except DistributionNotFound:
        return None



import pytest


def test_available(file_camera):
    available = file_camera.state.available
    assert available is not None
    assert isinstance(available, bool)


def test_connected(file_camera):
    connected = file_camera.state.connected
    assert connected is not None
    assert isinstance(connected, bool)


def test_temperature(file_camera):
    from zivid.camera_state import CameraState

    temperature = file_camera.state.temperature
    assert temperature is not None
    assert isinstance(temperature, CameraState.Temperature)


def test_temperature_dmd(file_camera):
    import numbers

    dmd = file_camera.state.temperature.dmd
    assert dmd is not None
    assert isinstance(dmd, numbers.Real)


def test_temperature_general(file_camera):
    import numbers

    general = file_camera.state.temperature.general
    assert general is not None
    assert isinstance(general, numbers.Real)


def test_temperature_led(file_camera):
    import numbers

    led = file_camera.state.temperature.led
    assert led is not None
    assert isinstance(led, numbers.Real)


def test_temperature_lens(file_camera):
    import numbers

    lens = file_camera.state.temperature.lens
    assert lens is not None
    assert isinstance(lens, numbers.Real)


def test_temperature_pcb(file_camera):
    import numbers

    pcb = file_camera.state.temperature.pcb
    assert pcb is not None
    assert isinstance(pcb, numbers.Real)


def test_illegal_set_state(file_camera):

    with pytest.raises(AttributeError):
        file_camera.state = file_camera.state


def test_equal_state():
    from zivid.camera_state import CameraState

    state1 = CameraState(available=True)
    state2 = CameraState(available=True)

    assert state1 == state2


def test_not_equal_state():
    from zivid.camera_state import CameraState

    state1 = CameraState(available=True)
    state2 = CameraState(available=False)

    assert state1 != state2


def test_equal_temperature():
    from zivid.camera_state import CameraState

    temperature1 = CameraState.Temperature(dmd=33)
    temperature2 = CameraState.Temperature(dmd=33)

    assert temperature1 == temperature2


def test_not_equal_temperature():
    from zivid.camera_state import CameraState

    temperature1 = CameraState.Temperature(dmd=33)
    temperature2 = CameraState.Temperature(dmd=44)

    assert temperature1 != temperature2



"""Contains the PointCloud class."""
import numpy

import _zivid


class PointCloud:
    """Point cloud with x, y, z, RGB and color laid out on a 2D grid.

    An instance of this class is a handle to a point cloud stored on the compute device memory.
    Use the method copy_data to copy point cloud data from the compute device to a numpy
    array in host memory. Several formats are available.
    """

    class Downsampling:  # pylint: disable=too-few-public-methods
        """Collection of valid options to PointCloud.downsample()."""

        by2x2 = "by2x2"
        by3x3 = "by3x3"
        by4x4 = "by4x4"

        _valid_values = {
            "by2x2": _zivid.PointCloud.Downsampling.by2x2,
            "by3x3": _zivid.PointCloud.Downsampling.by3x3,
            "by4x4": _zivid.PointCloud.Downsampling.by4x4,
        }

        @classmethod
        def valid_values(cls):
            """Get list of allowed values.

            Returns:
                List of strings
            """
            return list(cls._valid_values.keys())

    def __init__(self, impl):
        """Initialize PointCloud wrapper.

        This constructor is only used internally, and should not be called by the end-user.

        Args:
            impl:   Reference to internal/back-end instance.

        Raises:
            TypeError: If argument does not match the expected internal class.
        """
        if not isinstance(impl, _zivid.PointCloud):
            raise TypeError(
                "Unsupported type for argument impl. Got {}, expected {}".format(
                    type(impl), type(_zivid.PointCloud)
                )
            )
        self.__impl = impl

    def copy_data(self, data_format):
        """Copy point cloud data from GPU to numpy array.

        Supported data formats:
        xyz:        ndarray(Height,Width,3) of float
        xyzw:       ndarray(Height,Width,4) of float
        z:          ndarray(Height,Width)   of float
        rgba:       ndarray(Height,Width,4) of uint8
        normals:    ndarray(Height,Width,3) of float
        snr:        ndarray(Height,Width)   of float
        xyzrgba:    ndarray(Height,Width)   of composite dtype (accessed with e.g. arr["x"])

        Args:
            data_format: A string specifying the data to be copied

        Returns:
            A numpy array with the requested data.

        Raises:
            ValueError: if the requested data format does not exist
        """
        self.__impl.assert_not_released()

        data_formats = {
            "xyz": _zivid.Array2DPointXYZ,
            "xyzw": _zivid.Array2DPointXYZW,
            "z": _zivid.Array2DPointZ,
            "rgba": _zivid.Array2DColorRGBA,
            "normals": _zivid.Array2DNormalXYZ,
            "snr": _zivid.Array2DSNR,
            "xyzrgba": _zivid.Array2DPointXYZColorRGBA,
        }
        try:
            data_format_class = data_formats[data_format]
        except KeyError as ex:
            raise ValueError(
                "Unsupported data format: {data_format}. Supported formats: {all_formats}".format(
                    data_format=data_format, all_formats=list(data_formats.keys())
                )
            ) from ex
        return numpy.array(data_format_class(self.__impl))

    def transform(self, matrix):
        """Transform the point cloud in-place by a 4x4 transformation matrix.

        The transform matrix must be affine, i.e., the last row of the matrix should be [0, 0, 0, 1].

        Args:
            matrix: A 4x4 numpy arrays of floats

        Returns:
            Reference to the same PointCloud instance (for chaining calls)
        """
        self.__impl.transform(matrix)
        return self

    def downsample(self, downsampling):
        """Downsample the point cloud in-place.

        Args:
            downsampling: One of the strings in PointCloud.Downsample.valid_values()

        Returns:
            Reference to the same PointCloud instance (for chaining calls)
        """
        internal_downsampling = (
            PointCloud.Downsampling._valid_values[  # pylint: disable=protected-access
                downsampling
            ]
        )
        self.__impl.downsample(internal_downsampling)
        return self

    def downsampled(self, downsampling):
        """Get a downsampled copy of the point cloud.

        Args:
            downsampling: One of the strings in PointCloud.Downsample.valid_values()

        Returns:
            A new PointCloud instance
        """
        internal_downsampling = (
            PointCloud.Downsampling._valid_values[  # pylint: disable=protected-access
                downsampling
            ]
        )
        return PointCloud(self.__impl.downsampled(internal_downsampling))

    @property
    def height(self):
        """Get the height of the point cloud (number of rows).

        Returns:
            A positive integer
        """
        return self.__impl.height()

    @property
    def width(self):
        """Get the width of the point cloud (number of columns).

        Returns:
            A positive integer
        """
        return self.__impl.width()

    def release(self):
        """Release the underlying resources."""
        try:
            impl = self.__impl
        except AttributeError:
            pass
        else:
            impl.release()

    def __enter__(self):
        return self

    def __exit__(self, exception_type, exception_value, traceback):
        self.release()

    def __del__(self):
        self.release()



import pytest


def test_one_acquisition_in_list(file_camera):
    import zivid

    acquisitions = [zivid.Settings.Acquisition()]
    settings = zivid.Settings(acquisitions=acquisitions)
    assert isinstance(acquisitions, list)
    with file_camera.capture(settings) as frame:
        assert frame
        assert isinstance(frame, zivid.frame.Frame)


def test_five_acquisitions_in_list(file_camera):
    import zivid

    acquisitions = [
        zivid.Settings.Acquisition(),
        zivid.Settings.Acquisition(),
        zivid.Settings.Acquisition(),
        zivid.Settings.Acquisition(),
        zivid.Settings.Acquisition(),
    ]
    settings = zivid.Settings(acquisitions=acquisitions)
    assert isinstance(acquisitions, list)
    with file_camera.capture(settings) as frame:
        assert frame
        assert isinstance(frame, zivid.frame.Frame)


def test_one_acquisition_in_tuple(file_camera):
    import zivid

    acquisitions = (zivid.Settings.Acquisition(),)
    settings = zivid.Settings(acquisitions=acquisitions)
    assert isinstance(acquisitions, tuple)
    with file_camera.capture(settings) as frame:
        assert frame
        assert isinstance(frame, zivid.frame.Frame)


def test_five_acquisition_in_tuple(file_camera):
    import zivid

    acquisitions = (
        zivid.Settings.Acquisition(),
        zivid.Settings.Acquisition(),
        zivid.Settings.Acquisition(),
        zivid.Settings.Acquisition(),
        zivid.Settings.Acquisition(),
    )
    settings = zivid.Settings(acquisitions=acquisitions)
    assert isinstance(acquisitions, tuple)
    with file_camera.capture(settings) as frame:
        assert frame
        assert isinstance(frame, zivid.frame.Frame)


def test_illegal_settings(file_camera):
    import zivid

    with pytest.raises(RuntimeError):
        file_camera.capture(zivid.Settings())

    with pytest.raises(TypeError):
        file_camera.capture([1, 2, 3, 4, 5])

    with pytest.raises(TypeError):
        file_camera.capture([zivid.Settings(), zivid.Settings(), 3])

    with pytest.raises(TypeError):
        file_camera.capture(file_camera.capture())


def test_empty_settings_list(file_camera):
    import _zivid

    with pytest.raises(TypeError):
        file_camera.capture([])


def test_diagnostics_capture(file_camera):
    from pathlib import Path
    from tempfile import TemporaryDirectory
    import os.path
    import zivid

    settings = zivid.Settings()
    settings.acquisitions.append(zivid.Settings.Acquisition())
    settings.diagnostics.enabled = False
    frame_diagnostics_off = file_camera.capture(settings)
    settings.diagnostics.enabled = True
    frame_diagnostics_on = file_camera.capture(settings)

    with TemporaryDirectory() as tmpdir:
        file_diagnostics_off = Path(tmpdir) / "diagnostics_off.zdf"
        file_diagnostics_on = Path(tmpdir) / "diagnostics_on.zdf"
        frame_diagnostics_off.save(file_diagnostics_off)
        frame_diagnostics_on.save(file_diagnostics_on)

        # Diagnostics ON should lead to a significantly larger file size.
        mb_diagnostics_off = os.path.getsize(str(file_diagnostics_off)) / 1e6
        mb_diagnostics_on = os.path.getsize(str(file_diagnostics_on)) / 1e6
        assert mb_diagnostics_on > (mb_diagnostics_off + 1.0)



import pytest


@pytest.mark.physical_camera
def test_copy_data(physical_camera_image_2d):
    import numpy as np

    image = physical_camera_image_2d
    rgba = image.copy_data()
    assert rgba is not None
    assert isinstance(rgba, np.ndarray)
    assert rgba.shape == (image.height, image.width, 4)
    assert rgba.dtype == np.uint8


@pytest.mark.physical_camera
def test_save_path(physical_camera_image_2d):
    from pathlib import Path

    physical_camera_image_2d.save(Path("some_file.png"))


@pytest.mark.physical_camera
def test_save_string(physical_camera_image_2d):
    physical_camera_image_2d.save("some_file.png")


@pytest.mark.physical_camera
def test_to_array_context_manager(physical_camera_frame_2d):
    with physical_camera_frame_2d.image_rgba() as image_2d:
        image_2d.copy_data()
    with pytest.raises(RuntimeError):
        image_2d.copy_data()


@pytest.mark.physical_camera
def test_save_context_manager(physical_camera_frame_2d):
    with physical_camera_frame_2d.image_rgba() as image_2d:
        image_2d.save("some_file.png")
    with pytest.raises(RuntimeError):
        image_2d.save("some_file.png")


@pytest.mark.physical_camera
def test_height(physical_camera_image_2d):
    height = physical_camera_image_2d.height

    assert height is not None
    assert isinstance(height, int)


@pytest.mark.physical_camera
def test_width(physical_camera_image_2d):
    width = physical_camera_image_2d.width

    assert width is not None
    assert isinstance(width, int)



"""Contains a the Image class."""
import numpy

import _zivid


class Image:
    """A two-dimensional image stored on the host."""

    def __init__(self, impl):
        """Initialize Image wrapper.

        This constructor is only used internally, and should not be called by the end-user.

        Args:
            impl:   Reference to internal/back-end instance.

        Raises:
            TypeError: If argument does not match the expected internal class.
        """
        if not isinstance(impl, _zivid.ImageRGBA):
            raise TypeError(
                "Unsupported type for argument impl. Got {}, expected {}".format(
                    type(impl), type(_zivid.ImageRGBA)
                )
            )
        self.__impl = impl

    @property
    def height(self):
        """Get the height of the image (number of rows).

        Returns:
            A positive integer
        """
        return self.__impl.height()

    @property
    def width(self):
        """Get the width of the image (number of columns).

        Returns:
            A positive integer
        """
        return self.__impl.width()

    def save(self, file_path):
        """Save the image to a file.

        The supported file type is PNG with extension .png.
        This method will throw an exception if failing to save to the provided file_path.

        Args:
            file_path: A pathlib.Path instance or a string specifying destination
        """
        self.__impl.save(str(file_path))

    def copy_data(self):
        """Copy image data to numpy array.

        Returns:
            A numpy array containing color pixel data
        """
        self.__impl.assert_not_released()
        return numpy.array(self.__impl)

    def release(self):
        """Release the underlying resources."""
        try:
            impl = self.__impl
        except AttributeError:
            pass
        else:
            impl.release()

    def __enter__(self):
        return self

    def __exit__(self, exception_type, exception_value, traceback):
        self.release()

    def __del__(self):
        self.release()



"""Module for calibration features, such as HandEye and MultiCamera."""
# pylint: disable=unused-import
from zivid._calibration.detector import DetectionResult, detect_feature_points
from zivid._calibration.hand_eye import (
    HandEyeInput,
    HandEyeResidual,
    HandEyeOutput,
    calibrate_eye_in_hand,
    calibrate_eye_to_hand,
)
from zivid._calibration.multi_camera import (
    MultiCameraResidual,
    MultiCameraOutput,
    calibrate_multi_camera,
)
from zivid._calibration.pose import Pose



"""Module containing implementation of feature point detection functionality.

This module should not be imported directly by end-user, but rather accessed through
the zivid.calibration module.
"""
import _zivid


class DetectionResult:
    """Class representing detected feature points."""

    def __init__(self, impl):
        """Initialize DetectionResult wrapper.

        This constructor is only used internally, and should not be called by the end-user.

        Args:
            impl:   Reference to internal/back-end instance.

        Raises:
            TypeError: If argument does not match the expected internal class.
        """
        if not isinstance(impl, _zivid.calibration.DetectionResult):
            raise TypeError(
                "Unsupported type for argument impl. Got {}, expected {}".format(
                    type(impl), type(_zivid.calibration.DetectionResult)
                )
            )

        self.__impl = impl

    def valid(self):
        """Check validity of DetectionResult.

        Returns:
            True if DetectionResult is valid
        """
        return self.__impl.valid()

    def centroid(self):
        """Get the centroid of the detected feature points.

        Will throw an exception if the DetectionResult is not valid.

        Returns:
            A 1D array containing the X, Y and Z coordinates of the centroid
        """
        return self.__impl.centroid()

    def __bool__(self):
        return bool(self.__impl)

    def __str__(self):
        return str(self.__impl)


def detect_feature_points(point_cloud):
    """Detect feature points from a calibration object in a point cloud.

    Args:
        point_cloud: PointCloud containing a calibration object

    Returns:
        A DetectionResult instance
    """

    return DetectionResult(
        _zivid.calibration.detect_feature_points(
            point_cloud._PointCloud__impl  # pylint: disable=protected-access
        )
    )



"""Contains the Frame class."""
from pathlib import Path

import _zivid
from zivid.settings import _to_settings
from zivid.camera_info import _to_camera_info
from zivid.camera_state import _to_camera_state
from zivid.frame_info import _to_frame_info
from zivid.point_cloud import PointCloud


class Frame:
    """A frame captured by a Zivid camera.

    Contains the point cloud (stored on compute device memory) as well as
    calibration data, settings and state used by the API at time of the frame
    capture. Use the point_cloud() method to access point cloud data.
    """

    def __init__(self, file_name):
        """Create a frame by loading data from a file.

        Args:
            file_name: A pathlib.Path instance or a string specifying a Zivid Data File (ZDF)

        Raises:
            TypeError: Unsupported type provided for file name
        """
        if isinstance(file_name, (str, Path)):
            self.__impl = _zivid.Frame(str(file_name))
        elif isinstance(file_name, _zivid.Frame):
            self.__impl = file_name
        else:
            raise TypeError(
                "Unsupported type for argument file_name. Got {}, expected {} or {}.".format(
                    type(file_name), str.__name__, Path.__name__
                )
            )

    def __str__(self):
        return str(self.__impl)

    def point_cloud(self):
        """Get the point cloud.

        See documentation/functions of zivid.PointCloud for instructions on how to
        retrieve point cloud data on various formats.

        Returns:
            A PointCloud instance
        """
        return PointCloud(self.__impl.point_cloud())

    def save(self, file_path):
        """Save the frame to file. The file type is determined from the file extension.

        Args:
            file_path: A pathlib.Path instance or a string specifying destination

        """
        self.__impl.save(str(file_path))

    def load(self, file_path):
        """Load a frame from a Zivid data file.

        Args:
            file_path: A pathlib.Path instance or a string specifying a ZDF file to load
        """
        self.__impl.load(str(file_path))

    @property
    def settings(self):
        """Get the settings used to capture this frame.

        Returns:
            A Settings instance
        """
        return _to_settings(self.__impl.settings)

    @property
    def state(self):
        """Get the camera state data at the time of the frame capture.

        Returns:
            A CameraState instance
        """
        return _to_camera_state(self.__impl.state)

    @property
    def info(self):
        """Get information collected at the time of the frame capture.

        Returns:
            A FrameInfo instance
        """
        return _to_frame_info(self.__impl.info)

    @property
    def camera_info(self):
        """Get information about the camera used to capture the frame.

        Returns:
            A CameraInfo instance
        """
        return _to_camera_info(self.__impl.camera_info)

    def release(self):
        """Release the underlying resources."""
        try:
            impl = self.__impl
        except AttributeError:
            pass
        else:
            impl.release()

    def __enter__(self):
        return self

    def __exit__(self, exception_type, exception_value, traceback):
        self.release()

    def __del__(self):
        self.release()
